@@ -7,12 +7,12 @@ use std::sync::atomic::{AtomicU32, Ordering};
 
 use crate::archetype::{Archetype, Comp, CompMut, TypeMetadataSet};
 
-/// The ECS world storing [Entity] and components.
+/// The ECS world storing [Entities](Entity) and components.
 pub struct World {
     tag: u32,
-    entities: Vec<EntityMetadata>,
+    pub(crate) entities: Vec<EntityMetadata>,
     free_list: Vec<u32>,
-    archetypes: Vec<Archetype>,
+    pub(crate) archetypes: Vec<Archetype>,
     insert_map: HashMap<(u32, TypeId), u32, BuildHasherDefault<IndexTypeIdHasher>>,
     remove_map: HashMap<(u32, TypeId), u32, BuildHasherDefault<IndexTypeIdHasher>>,
 }
@@ -128,10 +128,6 @@ impl World {
         (self.tag, self.archetypes.len() as u32)
     }
 
-    pub(crate) fn archetypes(&self) -> &[Archetype] {
-        &self.archetypes
-    }
-
     /// Insert components for a given [Entity].
     ///
     /// # Panics
@@ -261,6 +257,25 @@ impl World {
 }
 
 impl World {
+    /// Check if a given [Entity] exists.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use rs_ecs::*;
+    /// let mut world = World::new();
+    ///
+    /// let entity = world.alloc();
+    /// assert!(world.exists(entity));
+    ///
+    /// world.free(entity);
+    /// assert!(!world.exists(entity));
+    /// ```
+    pub fn exists(&self, ent: Entity) -> bool {
+        let meta = &self.entities[ent.id as usize];
+        ent.gen == meta.gen
+    }
+
     /// Check if a certain component type is present for an [Entity].
     ///
     /// # Example
@@ -286,6 +301,8 @@ impl World {
 
     /// Get an immutable reference to the component of the given type for an [Entity].
     ///
+    /// Note that for repeated calls, [map](crate::QueryRef::map) can be used to amortize the set-up costs.
+    ///
     /// # Example
     ///
     /// ```
@@ -309,6 +326,8 @@ impl World {
 
     /// Get a mutable reference to the component of the given type for an [Entity].
     ///
+    /// Note that for repeated calls, [map](crate::QueryRef::map) can be used to amortize the set-up costs.
+    ///
     /// # Example
     ///
     /// ```
@@ -338,15 +357,15 @@ impl World {
 /// An opaque entity identifier.
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct Entity {
-    id: u32,
-    gen: NonZeroU32,
+    pub(crate) id: u32,
+    pub(crate) gen: NonZeroU32,
 }
 
 #[derive(Clone, Copy)]
-struct EntityMetadata {
-    gen: NonZeroU32,
-    ty: u32,
-    idx: u32,
+pub struct EntityMetadata {
+    pub gen: NonZeroU32,
+    pub ty: u32,
+    pub idx: u32,
 }
 
 impl Default for EntityMetadata {
